package edu.columbia.sip;

import mjs.util.Debug;
import java.net.Socket;

/** class SIPMessage encapsulates the contents of a single SIP message.  It is created
 * by SIPParser and returned via getMessage().
 *
 * @author Moshe Sambol
 */
public class SIPMessage {
    public static final int REQUEST = 1;
    public static final int RESPONSE = 2;
    
    private StringBuffer _headers;
    private StringBuffer _contents;
        
    /** Constructor.
     */
    public SIPMessage() {
        _headers = new StringBuffer();
        _contents = new StringBuffer();
    }
   
    /** get the headers of the SIP message
     *
     * @returns a String representing all of the headers, newline separated.
     */
    public String getHeaders() {
        return _headers.toString();
    }
    
    /** set the headers of the SIP message
     *
     * @param a String representing all of the headers, newline separated.
     */
    public void setHeaders(String h) {
        _headers = new StringBuffer(h);
    }
    
    /** add a header value - useful for multiple header values (eg. Via's)
     *
     */
    public void addHeader(String key, String value) {
       // for efficiency should do this with string buffers instead of strings
       _headers.append(key);
       _headers.append(": ");
       _headers.append(value);
       _headers.append("\n");
    }
    
    /** add a header value */
    public void addHeader(ProtocolToken key, String value) {
        addHeader(key.toString(), value);
    }
    
    /** add a header value */
    public void addHeader(ProtocolToken key, SIPURL value) {
        addHeader(key.toString(), value.toString());
    }
    
    /** set a single header value */
    public void setHeader(ProtocolToken key, SIPURL value) {
        setHeader(key.toString(), value.toString());
    }
    
    /** set a single header value
     * 
     */
    public void setHeader(ProtocolToken key, String value) {
        setHeader(key.toString(), value);
    }

    /** set a single header value
     *
     */
    public void setHeader(String key, String value) {
     //   System.out.println("SIPMessage.setHeader(" + key + ") called with " + value);
        int startpos = -1;
        int endpos = 0;
        String hstr = _headers.toString();

        // special case: is it the first header?
        startpos = hstr.indexOf(key + ": ");
        if (startpos != 0) {
            // other headers search
            startpos = hstr.indexOf("\n" + key + ": ");
            if (startpos != -1) {
                startpos++; // move past the newline
            }
        }
        if (startpos != -1) { // the header already exists
            endpos = hstr.indexOf("\n", startpos);
            _headers = new StringBuffer(hstr.substring(0, startpos));
            _headers.append(key);
            _headers.append(": ");
            _headers.append(value);
            _headers.append(hstr.substring(endpos));
        } else {
            // the header didn't already exist
            addHeader(key, value);
        }
    }


    /** set the contents of the SIP message
     *
     * @param a String representing all of the contents
     */
    public void setContents(String c) {
        _contents = new StringBuffer(c);
    }

    /** get the contents of the SIP message
     *
     * @returns a String representing the contents of the body
     * of the message.
     */
    public String getContents() {
        return _contents.toString();
    }

    public String getHeader(ProtocolToken key) {
        return getHeader(key.toString());
    } // end get(ProtocolToken key)

    public String getHeader(String key) {
        int startpos = -1;
        int endpos = 0;
        String hstr = _headers.toString();

        // NOTE: THIS METHOD WILL NEED TO BE REWORKED TO SUPPORT
        // THE getHeader("primary.secondary") FORMAT

        // special case: is it the first header?
        startpos = hstr.indexOf(key);
        if (startpos != 0) {
            // other headers search
            startpos = hstr.indexOf('\n' + key);
            if (startpos != -1) {
                startpos++; // move past the newline
            }
        }
        if (startpos != -1) {
            // we found this key value pair
            startpos = hstr.indexOf(SIPToken.SIPSEPARATOR.toString(), startpos) + 
                                    SIPToken.SIPSEPARATOR.toString().length();
            endpos = hstr.indexOf('\n', startpos);
            return hstr.substring(startpos, endpos);
        }
        System.out.println("SIPMessage.getHeader(" + key + ") that header not found in this message.");
        return null;
    }

    public static int getType(String head) throws SIPParseException {
   //     Debug.println("SIPMessage.getType() getting type of message with first header line = '" + head + "'");
        
        String firstfour = head.substring(0, 4);
        if (firstfour.equals("SIP/")) {
            return RESPONSE;
        } else if (SIPRequest.isRequestMethod(head.substring(0, head.indexOf(' ')))) { 
            return REQUEST;
        } else {
            throw new SIPParseException("SIPMessage.getType(): Unsupported status line in header: " +
                                    head.substring(0, head.indexOf("\n")));
        }
    } // end getType()

            
    /** return a correct Content-length line for this message.
     */
    public String getContentLengthLine() {
        int size = _contents.length();
        return SIPToken.CONTENT_LENGTH.toString() + 
            SIPToken.SIPSEPARATOR + String.valueOf(size) + '\n';
    }
    
    public String toString() {
        // this method shouldn't really be called - overriden by children
        // SIPRequest and SIPResponse
        return _headers.toString() + _contents.toString();
    }

} // end class SIPMessage