package edu.columbia.sip;

import mjs.util.Debug;

public class SIPRequest extends SIPMessage {
    private SIPToken _method;
    private SIPURL _url;
    
    /** the server to which a response should be sent - used to get port info, etc. */
  //  private SIPServer _server;

    /** the callID of this request */
    private String _callID;
    
    /** Constructor 
     * 
     * @param method one of the draft-defined SIP request methods
     * @param subjurl the SIP URL of the subject of this method 
     * @param myurl the SIP URL of the request sender (optional)
     *
     * @throws SIPException if the method is not a legitimate SIP request method
     */
    public SIPRequest(SIPToken method, SIPURL subjurl, SIPURL myurl) throws SIPException {
        _method = method;
        _url = subjurl;
  //      _server = null;
        if (myurl != null) {
            Debug.println("SIPRequest.CTOR() called to send message from " + myurl.toString());
        }
        
        if (!isRequestMethod(method)) {
            throw new SIPException("Cannot create a SIPRequest with unsupported method " 
            + method.toString());
        }
    
        _callID = SIPServer.getCallIDManager().generateCallID();
        SIPServer.getCallIDManager().associate(_callID, this);
        setHeader(SIPToken.CALL_ID, _callID);
        
        if (myurl != null) {
            setHeader(SIPToken.FROM, myurl);
        }
                
    } // end SIPRequest CTOR
    

    /** set the server to which responses should be sent. */
    public void respondToServer(SIPServer server) {
    //    _server = server;
      //     setHeader("Location", _url.toString() + ":" + server.getTCPPort() + ";transport=tcp");
         setHeader("Location", "sip:" + server.getAddress() + ":" + server.getTCPPort() + "; transport=tcp");
            
    }
    
    public String getResponseAddress() {
        String loc = getHeader(SIPToken.LOCATION);
        if (loc == null) {
            return null;
        } else {
            loc = loc.substring(4);  // skip the "sip:" prefix
            return loc.substring(0, loc.indexOf(':'));
        }
    }
    
    public int getResponsePort() {
        String loc = getHeader(SIPToken.LOCATION);
        if (loc == null) {
            return -1;
        } else {
            loc = loc.substring(4);  // skip the "sip:" prefix
            return Integer.parseInt(loc.substring(loc.indexOf(':')+1, loc.indexOf(';')));
        }
    }

    public String getDestinationAddress() {
        String loc = getHeader(SIPToken.TO);
        if (loc == null) {
            return null;
        } else {
            if (loc.indexOf("sip:") > -1) {
                loc = loc.substring(loc.indexOf("sip:") + 4);
            }
            if (loc.indexOf(':') > -1) {
                return loc.substring(0, loc.indexOf(':'));
            } else {
                return loc;
            }
        }
    }

    public int getDestinationPort() {
        String loc = getHeader(SIPToken.TO);
        if (loc == null) {
            return -1;
        } else {
            if (loc.indexOf("sip:") > -1) {
                loc = loc.substring(loc.indexOf("sip:") + 4);
            }
            if (loc.indexOf(':') > -1) {
                return Integer.parseInt(loc.substring(loc.indexOf(':') + 1));
            } else {
                return SIPServer.DEFAULT_PORT;
            }
        }
    }

    public SIPURL getDestinationURL() {
        Debug.println("SIPRequest.getDestinationURL()...");
        String loc = getHeader(SIPToken.TO);
        if (loc == null) {
            return null;
        } else {
            if (loc.indexOf("sip:") > -1) {
                return new SIPURL(loc);
            } else {
                return new SIPURL("sip:" + loc);
            }
        }
    }
        
    public SIPToken getMethod() {
        return _method;
    }
    
    public SIPURL getURL() {
        return _url;
    }
    
    public String toString() {
        // compose an *ordered* string which represents the SIPMessage 
        // that should be sent
        
        StringBuffer sb = new StringBuffer();
        sb.append(_method.toString());
        sb.append(' ');
        sb.append(_url.toString());
        sb.append(' ');
        sb.append(SIPToken.SIPVERSION);
        sb.append('\n');
    
        sb.append(getHeaders());
        sb.append(getContentLengthLine());
        sb.append('\n');
        sb.append(getContents());
        
        return sb.toString();
    } // end toString()

    /** check whether a given token represents a valid SIP request Method */
    public static boolean isRequestMethod(SIPToken t) {
        return isRequestMethod(t.toString());
    }

    /** check whether a given token represents a valid SIP request Method */
    public static boolean isRequestMethod(String s) {
//        System.out.println("SIPRequest.isRequestMethod() called with '" + s + "'");
        if (SIPToken.ACK.equals(s) || SIPToken.INVITE.equals(s) || 
        SIPToken.HERE.equals(s) || SIPToken.GONE.equals(s) || 
        SIPToken.OPTIONS.equals(s) || SIPToken.BYE.equals(s) || 
        SIPToken.CANCEL.equals(s) || SIPToken.REGISTER.equals(s)) {
            return true;
        } else {
            return false;
        }
    }

}  // end class SIPRequest
